@@ -22,6 +22,7 @@ import re
 import tempfile
 import time
 import unittest
+import mock
 
 import requests
 import six
@@ -33,7 +34,10 @@ from google.cloud.storage._helpers import _base64_md5hash
 from google.cloud.storage.bucket import LifecycleRuleDelete
 from google.cloud.storage.bucket import LifecycleRuleSetStorageClass
 from google.cloud import kms
+from google import resumable_media
+import google.auth
 import google.api_core
+from google.api_core import path_template
 import google.oauth2
 from test_utils.retry import RetryErrors
 from test_utils.system import unique_resource_id
@@ -191,14 +195,25 @@ class TestStorageBuckets(unittest.TestCase):
         self.assertEqual(created.storage_class, constants.ARCHIVE_STORAGE_CLASS)
 
     def test_lifecycle_rules(self):
+        import datetime
         from google.cloud.storage import constants
 
         new_bucket_name = "w-lifcycle-rules" + unique_resource_id("-")
+        custom_time_before = datetime.date(2018, 8, 1)
+        noncurrent_before = datetime.date(2018, 8, 1)
+
         self.assertRaises(
             exceptions.NotFound, Config.CLIENT.get_bucket, new_bucket_name
         )
         bucket = Config.CLIENT.bucket(new_bucket_name)
-        bucket.add_lifecycle_delete_rule(age=42)
+        bucket.add_lifecycle_delete_rule(
+            age=42,
+            number_of_newer_versions=3,
+            days_since_custom_time=2,
+            custom_time_before=custom_time_before,
+            days_since_noncurrent_time=2,
+            noncurrent_time_before=noncurrent_before,
+        )
         bucket.add_lifecycle_set_storage_class_rule(
             constants.COLDLINE_STORAGE_CLASS,
             is_live=False,
@@ -206,7 +221,14 @@ class TestStorageBuckets(unittest.TestCase):
         )
 
         expected_rules = [
-            LifecycleRuleDelete(age=42),
+            LifecycleRuleDelete(
+                age=42,
+                number_of_newer_versions=3,
+                days_since_custom_time=2,
+                custom_time_before=custom_time_before,
+                days_since_noncurrent_time=2,
+                noncurrent_time_before=noncurrent_before,
+            ),
             LifecycleRuleSetStorageClass(
                 constants.COLDLINE_STORAGE_CLASS,
                 is_live=False,
@@ -420,8 +442,8 @@ class TestStorageBuckets(unittest.TestCase):
             )
             to_delete.append(new_blob)
 
-            base_contents = blob.download_as_string()
-            copied_contents = new_blob.download_as_string()
+            base_contents = blob.download_as_bytes()
+            copied_contents = new_blob.download_as_bytes()
             self.assertEqual(base_contents, copied_contents)
         finally:
             for blob in to_delete:
@@ -450,8 +472,8 @@ class TestStorageBuckets(unittest.TestCase):
             )
             to_delete.append(new_blob)
 
-            base_contents = blob.download_as_string()
-            copied_contents = new_blob.download_as_string()
+            base_contents = blob.download_as_bytes()
+            copied_contents = new_blob.download_as_bytes()
             self.assertEqual(base_contents, copied_contents)
         finally:
             for blob in to_delete:
@@ -480,8 +502,8 @@ class TestStorageBuckets(unittest.TestCase):
             )
             to_delete.append(new_blob)
 
-            base_contents = blob.download_as_string()
-            copied_contents = new_blob.download_as_string()
+            base_contents = blob.download_as_bytes()
+            copied_contents = new_blob.download_as_bytes()
             self.assertEqual(base_contents, copied_contents)
         finally:
             for blob in to_delete:
@@ -507,7 +529,7 @@ class TestStorageBuckets(unittest.TestCase):
         to_add.upload_from_string(data)
         try:
             found = with_user_project.get_blob("blob-name")
-            self.assertEqual(found.download_as_string(), data)
+            self.assertEqual(found.download_as_bytes(), data)
         finally:
             to_add.delete()
 
@@ -554,6 +576,37 @@ class TestStorageWriteFiles(TestStorageFiles):
             md5_hash = md5_hash.encode("utf-8")
         self.assertEqual(md5_hash, file_data["hash"])
 
+    def test_large_file_write_from_stream_with_checksum(self):
+        blob = self.bucket.blob("LargeFile")
+
+        file_data = self.FILES["big"]
+        with open(file_data["path"], "rb") as file_obj:
+            blob.upload_from_file(file_obj, checksum="crc32c")
+            self.case_blobs_to_delete.append(blob)
+
+        md5_hash = blob.md5_hash
+        if not isinstance(md5_hash, six.binary_type):
+            md5_hash = md5_hash.encode("utf-8")
+        self.assertEqual(md5_hash, file_data["hash"])
+
+    def test_large_file_write_from_stream_with_failed_checksum(self):
+        blob = self.bucket.blob("LargeFile")
+
+        file_data = self.FILES["big"]
+
+        # Intercept the digest processing at the last stage and replace it with garbage.
+        # This is done with a patch to monkey-patch the resumable media library's checksum
+        # processing; it does not mock a remote interface like a unit test would. The
+        # remote API is still exercised.
+        with open(file_data["path"], "rb") as file_obj:
+            with mock.patch(
+                "google.resumable_media._helpers.prepare_checksum_digest",
+                return_value="FFFFFF==",
+            ):
+                with self.assertRaises(resumable_media.DataCorruption):
+                    blob.upload_from_file(file_obj, checksum="crc32c")
+                self.assertFalse(blob.exists())
+
     def test_large_encrypted_file_write_from_stream(self):
         blob = self.bucket.blob("LargeFile", encryption_key=self.ENCRYPTION_KEY)
 
@@ -567,12 +620,12 @@ class TestStorageWriteFiles(TestStorageFiles):
             md5_hash = md5_hash.encode("utf-8")
         self.assertEqual(md5_hash, file_data["hash"])
 
-        temp_filename = tempfile.mktemp()
-        with open(temp_filename, "wb") as file_obj:
-            blob.download_to_file(file_obj)
+        with tempfile.NamedTemporaryFile() as temp_f:
+            with open(temp_f.name, "wb") as file_obj:
+                blob.download_to_file(file_obj)
 
-        with open(temp_filename, "rb") as file_obj:
-            md5_temp_hash = _base64_md5hash(file_obj)
+            with open(temp_f.name, "rb") as file_obj:
+                md5_temp_hash = _base64_md5hash(file_obj)
 
         self.assertEqual(md5_temp_hash, file_data["hash"])
 
@@ -588,6 +641,32 @@ class TestStorageWriteFiles(TestStorageFiles):
             md5_hash = md5_hash.encode("utf-8")
         self.assertEqual(md5_hash, file_data["hash"])
 
+    def test_small_file_write_from_filename_with_checksum(self):
+        blob = self.bucket.blob("SmallFile")
+
+        file_data = self.FILES["simple"]
+        blob.upload_from_filename(file_data["path"], checksum="crc32c")
+        self.case_blobs_to_delete.append(blob)
+
+        md5_hash = blob.md5_hash
+        if not isinstance(md5_hash, six.binary_type):
+            md5_hash = md5_hash.encode("utf-8")
+        self.assertEqual(md5_hash, file_data["hash"])
+
+    def test_small_file_write_from_filename_with_failed_checksum(self):
+        blob = self.bucket.blob("SmallFile")
+
+        file_data = self.FILES["simple"]
+        # Intercept the digest processing at the last stage and replace it with garbage
+        with mock.patch(
+            "google.resumable_media._helpers.prepare_checksum_digest",
+            return_value="FFFFFF==",
+        ):
+            with self.assertRaises(google.api_core.exceptions.BadRequest):
+                blob.upload_from_filename(file_data["path"], checksum="crc32c")
+
+        self.assertFalse(blob.exists())
+
     @unittest.skipUnless(USER_PROJECT, "USER_PROJECT not set in environment.")
     def test_crud_blob_w_user_project(self):
         with_user_project = Config.CLIENT.bucket(
@@ -622,8 +701,8 @@ class TestStorageWriteFiles(TestStorageFiles):
             blob.reload()
 
             # Exercise 'objects.get' (media) w/ userProject.
-            self.assertEqual(blob0.download_as_string(), file_contents)
-            self.assertEqual(blob1.download_as_string(), b"gen1")
+            self.assertEqual(blob0.download_as_bytes(), file_contents)
+            self.assertEqual(blob1.download_as_bytes(), b"gen1")
 
             # Exercise 'objects.patch' w/ userProject.
             blob0.content_language = "en"
@@ -683,10 +762,10 @@ class TestStorageWriteFiles(TestStorageFiles):
 
             # Exercise 'objects.get' (media) w/ generation match.
             self.assertEqual(
-                blob0.download_as_string(if_generation_match=gen0), file_contents
+                blob0.download_as_bytes(if_generation_match=gen0), file_contents
             )
             self.assertEqual(
-                blob1.download_as_string(if_generation_not_match=gen0), b"gen1"
+                blob1.download_as_bytes(if_generation_not_match=gen0), b"gen1"
             )
 
             # Exercise 'objects.patch' w/ generation match.
@@ -776,12 +855,14 @@ class TestStorageWriteFiles(TestStorageFiles):
 
         same_blob = self.bucket.blob("MyBuffer")
         same_blob.reload()  # Initialize properties.
-        temp_filename = tempfile.mktemp()
-        with open(temp_filename, "wb") as file_obj:
-            same_blob.download_to_file(file_obj)
 
-        with open(temp_filename, "rb") as file_obj:
-            stored_contents = file_obj.read()
+        with tempfile.NamedTemporaryFile() as temp_f:
+
+            with open(temp_f.name, "wb") as file_obj:
+                same_blob.download_to_file(file_obj)
+
+            with open(temp_f.name, "rb") as file_obj:
+                stored_contents = file_obj.read()
 
         self.assertEqual(file_contents, stored_contents)
 
@@ -795,24 +876,55 @@ class TestStorageWriteFiles(TestStorageFiles):
 
         same_blob = self.bucket.blob("MyBuffer")
         same_blob.reload()  # Initialize properties.
-        temp_filename = tempfile.mktemp()
-        with open(temp_filename, "wb") as file_obj:
-            with self.assertRaises(google.api_core.exceptions.PreconditionFailed):
+
+        with tempfile.NamedTemporaryFile() as temp_f:
+
+            with open(temp_f.name, "wb") as file_obj:
+                with self.assertRaises(google.api_core.exceptions.PreconditionFailed):
+                    same_blob.download_to_file(
+                        file_obj, if_generation_match=WRONG_GENERATION_NUMBER
+                    )
+
                 same_blob.download_to_file(
-                    file_obj, if_generation_match=WRONG_GENERATION_NUMBER
+                    file_obj,
+                    if_generation_match=blob.generation,
+                    if_metageneration_match=blob.metageneration,
                 )
 
-            same_blob.download_to_file(
-                file_obj,
-                if_generation_match=blob.generation,
-                if_metageneration_match=blob.metageneration,
-            )
-
-        with open(temp_filename, "rb") as file_obj:
-            stored_contents = file_obj.read()
+            with open(temp_f.name, "rb") as file_obj:
+                stored_contents = file_obj.read()
 
         self.assertEqual(file_contents, stored_contents)
 
+    def test_download_w_failed_crc32c_checksum(self):
+        blob = self.bucket.blob("FailedChecksumBlob")
+        file_contents = b"Hello World"
+        blob.upload_from_string(file_contents)
+        self.case_blobs_to_delete.append(blob)
+
+        with tempfile.NamedTemporaryFile() as temp_f:
+            # Intercept the digest processing at the last stage and replace it with garbage.
+            # This is done with a patch to monkey-patch the resumable media library's checksum
+            # processing; it does not mock a remote interface like a unit test would. The
+            # remote API is still exercised.
+            with mock.patch(
+                "google.resumable_media._helpers.prepare_checksum_digest",
+                return_value="FFFFFF==",
+            ):
+                with self.assertRaises(resumable_media.DataCorruption):
+                    blob.download_to_filename(temp_f.name, checksum="crc32c")
+
+                # Confirm the file was deleted on failure
+                self.assertFalse(os.path.isfile(temp_f.name))
+
+                # Now download with checksumming turned off
+                blob.download_to_filename(temp_f.name, checksum=None)
+
+            with open(temp_f.name, "rb") as file_obj:
+                stored_contents = file_obj.read()
+
+            self.assertEqual(file_contents, stored_contents)
+
     def test_copy_existing_file(self):
         filename = self.FILES["logo"]["path"]
         blob = storage.Blob("CloudLogo", bucket=self.bucket)
@@ -824,8 +936,8 @@ class TestStorageWriteFiles(TestStorageFiles):
         )
         self.case_blobs_to_delete.append(new_blob)
 
-        base_contents = blob.download_as_string()
-        copied_contents = new_blob.download_as_string()
+        base_contents = blob.download_as_bytes()
+        copied_contents = new_blob.download_as_bytes()
         self.assertEqual(base_contents, copied_contents)
 
     def test_download_blob_w_uri(self):
@@ -834,15 +946,25 @@ class TestStorageWriteFiles(TestStorageFiles):
         blob.upload_from_string(file_contents)
         self.case_blobs_to_delete.append(blob)
 
-        temp_filename = tempfile.mktemp()
-        with open(temp_filename, "wb") as file_obj:
-            Config.CLIENT.download_blob_to_file(
-                "gs://" + self.bucket.name + "/MyBuffer", file_obj
-            )
+        with tempfile.NamedTemporaryFile() as temp_f:
 
-        with open(temp_filename, "rb") as file_obj:
-            stored_contents = file_obj.read()
+            with open(temp_f.name, "wb") as file_obj:
+                Config.CLIENT.download_blob_to_file(
+                    "gs://" + self.bucket.name + "/MyBuffer", file_obj
+                )
 
+            with open(temp_f.name, "rb") as file_obj:
+                stored_contents = file_obj.read()
+
+        self.assertEqual(file_contents, stored_contents)
+
+    def test_download_blob_as_text(self):
+        blob = self.bucket.blob("MyBuffer")
+        file_contents = "Hello World"
+        blob.upload_from_string(file_contents)
+        self.case_blobs_to_delete.append(blob)
+
+        stored_contents = blob.download_as_text()
         self.assertEqual(file_contents, stored_contents)
 
     def test_upload_gzip_encoded_download_raw(self):
@@ -856,10 +978,10 @@ class TestStorageWriteFiles(TestStorageFiles):
         blob.content_encoding = "gzip"
         blob.upload_from_file(raw_stream, rewind=True)
 
-        expanded = blob.download_as_string()
+        expanded = blob.download_as_bytes()
         self.assertEqual(expanded, payload)
 
-        raw = blob.download_as_string(raw_download=True)
+        raw = blob.download_as_bytes(raw_download=True)
         self.assertEqual(raw, zipped)
 
     def test_resumable_upload_with_generation_match(self):
@@ -900,6 +1022,18 @@ class TestStorageWriteFiles(TestStorageFiles):
         owner = same_blob.owner
         self.assertIn(user_email, owner["entity"])
 
+    def test_blob_crc32_md5_hash(self):
+        blob = self.bucket.blob("MyBuffer")
+        file_contents = b"Hello World"
+        blob.upload_from_string(file_contents)
+        self.case_blobs_to_delete.append(blob)
+
+        download_blob = self.bucket.blob("MyBuffer")
+
+        self.assertEqual(download_blob.download_as_string(), file_contents)
+        self.assertEqual(download_blob.crc32c, blob.crc32c)
+        self.assertEqual(download_blob.md5_hash, blob.md5_hash)
+
 
 class TestUnicode(unittest.TestCase):
     @vpcsc_config.skip_if_inside_vpcsc
@@ -919,7 +1053,7 @@ class TestUnicode(unittest.TestCase):
         for blob_name, file_contents in test_data.items():
             blob = bucket.blob(blob_name)
             self.assertEqual(blob.name, blob_name)
-            self.assertEqual(blob.download_as_string(), file_contents)
+            self.assertEqual(blob.download_as_bytes(), file_contents)
 
 
 class TestStorageListFiles(TestStorageFiles):
@@ -1280,12 +1414,16 @@ class TestStorageSignURLs(unittest.TestCase):
     def test_create_signed_read_url_v2_w_access_token(self):
         client = iam_credentials_v1.IAMCredentialsClient()
         service_account_email = Config.CLIENT._credentials.service_account_email
-        name = client.service_account_path("-", service_account_email)
+        name = path_template.expand(
+            "projects/{project}/serviceAccounts/{service_account}",
+            project="-",
+            service_account=service_account_email,
+        )
         scope = [
             "https://www.googleapis.com/auth/devstorage.read_write",
             "https://www.googleapis.com/auth/iam",
         ]
-        response = client.generate_access_token(name, scope)
+        response = client.generate_access_token(name=name, scope=scope)
         self._create_signed_read_url_helper(
             service_account_email=service_account_email,
             access_token=response.access_token,
@@ -1294,12 +1432,16 @@ class TestStorageSignURLs(unittest.TestCase):
     def test_create_signed_read_url_v4_w_access_token(self):
         client = iam_credentials_v1.IAMCredentialsClient()
         service_account_email = Config.CLIENT._credentials.service_account_email
-        name = client.service_account_path("-", service_account_email)
+        name = path_template.expand(
+            "projects/{project}/serviceAccounts/{service_account}",
+            project="-",
+            service_account=service_account_email,
+        )
         scope = [
             "https://www.googleapis.com/auth/devstorage.read_write",
             "https://www.googleapis.com/auth/iam",
         ]
-        response = client.generate_access_token(name, scope)
+        response = client.generate_access_token(name=name, scope=scope)
         self._create_signed_read_url_helper(
             version="v4",
             service_account_email=service_account_email,
@@ -1401,7 +1543,7 @@ class TestStorageCompose(TestStorageFiles):
         destination.compose([source_1, source_2])
         self.case_blobs_to_delete.append(destination)
 
-        composed = destination.download_as_string()
+        composed = destination.download_as_bytes()
         self.assertEqual(composed, SOURCE_1 + SOURCE_2)
 
     def test_compose_create_new_blob_wo_content_type(self):
@@ -1421,7 +1563,7 @@ class TestStorageCompose(TestStorageFiles):
         self.case_blobs_to_delete.append(destination)
 
         self.assertIsNone(destination.content_type)
-        composed = destination.download_as_string()
+        composed = destination.download_as_bytes()
         self.assertEqual(composed, SOURCE_1 + SOURCE_2)
 
     def test_compose_replace_existing_blob(self):
@@ -1438,7 +1580,7 @@ class TestStorageCompose(TestStorageFiles):
 
         original.compose([original, to_append])
 
-        composed = original.download_as_string()
+        composed = original.download_as_bytes()
         self.assertEqual(composed, BEFORE + TO_APPEND)
 
     def test_compose_with_generation_match(self):
@@ -1466,7 +1608,7 @@ class TestStorageCompose(TestStorageFiles):
             if_metageneration_match=[original.metageneration, to_append.metageneration],
         )
 
-        composed = original.download_as_string()
+        composed = original.download_as_bytes()
         self.assertEqual(composed, BEFORE + TO_APPEND)
 
     @unittest.skipUnless(USER_PROJECT, "USER_PROJECT not set in environment.")
@@ -1492,7 +1634,7 @@ class TestStorageCompose(TestStorageFiles):
             destination.content_type = "text/plain"
             destination.compose([source_1, source_2])
 
-            composed = destination.download_as_string()
+            composed = destination.download_as_bytes()
             self.assertEqual(composed, SOURCE_1 + SOURCE_2)
         finally:
             retry_429_harder(created.delete)(force=True)
@@ -1508,7 +1650,7 @@ class TestStorageRewrite(TestStorageFiles):
         source = self.bucket.blob("source")
         source.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(source)
-        source_data = source.download_as_string()
+        source_data = source.download_as_bytes()
 
         KEY = os.urandom(32)
         dest = self.bucket.blob("dest", encryption_key=KEY)
@@ -1519,7 +1661,7 @@ class TestStorageRewrite(TestStorageFiles):
         self.assertEqual(rewritten, len(source_data))
         self.assertEqual(total, len(source_data))
 
-        self.assertEqual(source.download_as_string(), dest.download_as_string())
+        self.assertEqual(source.download_as_bytes(), dest.download_as_bytes())
 
     def test_rewrite_rotate_encryption_key(self):
         BLOB_NAME = "rotating-keys"
@@ -1529,7 +1671,7 @@ class TestStorageRewrite(TestStorageFiles):
         source = self.bucket.blob(BLOB_NAME, encryption_key=SOURCE_KEY)
         source.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(source)
-        source_data = source.download_as_string()
+        source_data = source.download_as_bytes()
 
         DEST_KEY = os.urandom(32)
         dest = self.bucket.blob(BLOB_NAME, encryption_key=DEST_KEY)
@@ -1541,7 +1683,7 @@ class TestStorageRewrite(TestStorageFiles):
         self.assertEqual(rewritten, len(source_data))
         self.assertEqual(total, len(source_data))
 
-        self.assertEqual(dest.download_as_string(), source_data)
+        self.assertEqual(dest.download_as_bytes(), source_data)
 
     @unittest.skipUnless(USER_PROJECT, "USER_PROJECT not set in environment.")
     def test_rewrite_add_key_with_user_project(self):
@@ -1557,7 +1699,7 @@ class TestStorageRewrite(TestStorageFiles):
 
             source = with_user_project.blob("source")
             source.upload_from_filename(file_data["path"])
-            source_data = source.download_as_string()
+            source_data = source.download_as_bytes()
 
             KEY = os.urandom(32)
             dest = with_user_project.blob("dest", encryption_key=KEY)
@@ -1567,7 +1709,7 @@ class TestStorageRewrite(TestStorageFiles):
             self.assertEqual(rewritten, len(source_data))
             self.assertEqual(total, len(source_data))
 
-            self.assertEqual(source.download_as_string(), dest.download_as_string())
+            self.assertEqual(source.download_as_bytes(), dest.download_as_bytes())
         finally:
             retry_429_harder(created.delete)(force=True)
 
@@ -1587,7 +1729,7 @@ class TestStorageRewrite(TestStorageFiles):
             SOURCE_KEY = os.urandom(32)
             source = with_user_project.blob(BLOB_NAME, encryption_key=SOURCE_KEY)
             source.upload_from_filename(file_data["path"])
-            source_data = source.download_as_string()
+            source_data = source.download_as_bytes()
 
             DEST_KEY = os.urandom(32)
             dest = with_user_project.blob(BLOB_NAME, encryption_key=DEST_KEY)
@@ -1597,7 +1739,7 @@ class TestStorageRewrite(TestStorageFiles):
             self.assertEqual(rewritten, len(source_data))
             self.assertEqual(total, len(source_data))
 
-            self.assertEqual(dest.download_as_string(), source_data)
+            self.assertEqual(dest.download_as_bytes(), source_data)
         finally:
             retry_429_harder(created.delete)(force=True)
 
@@ -1613,7 +1755,7 @@ class TestStorageRewrite(TestStorageFiles):
 
             source = bucket.blob(BLOB_NAME)
             source.upload_from_filename(file_data["path"])
-            source_data = source.download_as_string()
+            source_data = source.download_as_bytes()
 
             dest = bucket.blob(BLOB_NAME)
 
@@ -1631,7 +1773,7 @@ class TestStorageRewrite(TestStorageFiles):
             self.assertEqual(token, None)
             self.assertEqual(rewritten, len(source_data))
             self.assertEqual(total, len(source_data))
-            self.assertEqual(dest.download_as_string(), source_data)
+            self.assertEqual(dest.download_as_bytes(), source_data)
         finally:
             retry_429_harder(created.delete)(force=True)
 
@@ -1898,13 +2040,14 @@ class TestKMSIntegration(TestStorageFiles):
         blob.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(blob)
         with open(file_data["path"], "rb") as _file_data:
-            self.assertEqual(blob.download_as_string(), _file_data.read())
+            self.assertEqual(blob.download_as_bytes(), _file_data.read())
         # We don't know the current version of the key.
         self.assertTrue(blob.kms_key_name.startswith(kms_key_name))
 
         (listed,) = list(self.bucket.list_blobs())
         self.assertTrue(listed.kms_key_name.startswith(kms_key_name))
 
+    @RetryErrors(unittest.TestCase.failureException)
     def test_bucket_w_default_kms_key_name(self):
         BLOB_NAME = "default-kms-key-name"
         OVERRIDE_BLOB_NAME = "override-default-kms-key-name"
@@ -1925,7 +2068,7 @@ class TestKMSIntegration(TestStorageFiles):
         defaulted_blob.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(defaulted_blob)
 
-        self.assertEqual(defaulted_blob.download_as_string(), contents)
+        self.assertEqual(defaulted_blob.download_as_bytes(), contents)
         # We don't know the current version of the key.
         self.assertTrue(defaulted_blob.kms_key_name.startswith(kms_key_name))
 
@@ -1937,7 +2080,7 @@ class TestKMSIntegration(TestStorageFiles):
         override_blob.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(override_blob)
 
-        self.assertEqual(override_blob.download_as_string(), contents)
+        self.assertEqual(override_blob.download_as_bytes(), contents)
         # We don't know the current version of the key.
         self.assertTrue(override_blob.kms_key_name.startswith(alt_kms_key_name))
 
@@ -1948,7 +2091,7 @@ class TestKMSIntegration(TestStorageFiles):
         alt_blob.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(alt_blob)
 
-        self.assertEqual(alt_blob.download_as_string(), contents)
+        self.assertEqual(alt_blob.download_as_bytes(), contents)
         # We don't know the current version of the key.
         self.assertTrue(alt_blob.kms_key_name.startswith(alt_kms_key_name))
 
@@ -1959,7 +2102,7 @@ class TestKMSIntegration(TestStorageFiles):
         cleartext_blob.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(cleartext_blob)
 
-        self.assertEqual(cleartext_blob.download_as_string(), contents)
+        self.assertEqual(cleartext_blob.download_as_bytes(), contents)
         self.assertIsNone(cleartext_blob.kms_key_name)
 
     def test_rewrite_rotate_csek_to_cmek(self):
@@ -1970,7 +2113,7 @@ class TestKMSIntegration(TestStorageFiles):
         source = self.bucket.blob(BLOB_NAME, encryption_key=SOURCE_KEY)
         source.upload_from_filename(file_data["path"])
         self.case_blobs_to_delete.append(source)
-        source_data = source.download_as_string()
+        source_data = source.download_as_bytes()
 
         kms_key_name = self._kms_key_name()
 
@@ -1992,7 +2135,7 @@ class TestKMSIntegration(TestStorageFiles):
         self.assertEqual(rewritten, len(source_data))
         self.assertEqual(total, len(source_data))
 
-        self.assertEqual(dest.download_as_string(), source_data)
+        self.assertEqual(dest.download_as_bytes(), source_data)
 
     def test_upload_new_blob_w_bucket_cmek_enabled(self):
         blob_name = "test-blob"
@@ -2006,16 +2149,17 @@ class TestKMSIntegration(TestStorageFiles):
 
         blob = self.bucket.blob(blob_name)
         blob.upload_from_string(payload)
+        retry_429_harder(blob.reload)()
         # We don't know the current version of the key.
         self.assertTrue(blob.kms_key_name.startswith(kms_key_name))
 
         blob.upload_from_string(alt_payload, if_generation_match=blob.generation)
         self.case_blobs_to_delete.append(blob)
 
-        self.assertEqual(blob.download_as_string(), alt_payload)
+        self.assertEqual(blob.download_as_bytes(), alt_payload)
 
         self.bucket.default_kms_key_name = None
-        self.bucket.patch()
+        retry_429_harder(self.bucket.patch)()
         self.assertIsNone(self.bucket.default_kms_key_name)
 
 
@@ -2240,7 +2384,7 @@ class TestIAMConfiguration(unittest.TestCase):
         blob.upload_from_string(payload)
 
         found = bucket.get_blob(blob_name)
-        self.assertEqual(found.download_as_string(), payload)
+        self.assertEqual(found.download_as_bytes(), payload)
 
         blob_acl = blob.acl
         with self.assertRaises(exceptions.BadRequest):
@@ -2318,7 +2462,7 @@ class TestV4POSTPolicies(unittest.TestCase):
                 {"bucket": bucket_name},
                 ["starts-with", "$Content-Type", "text/pla"],
             ],
-            expiration=datetime.datetime.now() + datetime.timedelta(hours=1),
+            expiration=datetime.datetime.utcnow() + datetime.timedelta(hours=1),
             fields={"content-type": "text/plain"},
         )
         with open(blob_name, "r") as f:
@@ -2345,7 +2489,7 @@ class TestV4POSTPolicies(unittest.TestCase):
                 {"bucket": bucket_name},
                 ["starts-with", "$Content-Type", "text/pla"],
             ],
-            expiration=datetime.datetime.now() + datetime.timedelta(hours=1),
+            expiration=datetime.datetime.utcnow() + datetime.timedelta(hours=1),
             fields={"x-goog-random": "invalid_field", "content-type": "text/plain"},
         )
         with open(blob_name, "r") as f:
